##############################################################################
#
# Copyright (c) 2002, 2003 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Zope Object Database: object database and persistence

The Zope Object Database provides an object-oriented database for
Python that provides a high-degree of transparency. Applications can
take advantage of object database features with few, if any, changes
to application logic.  ZODB includes features such as a pluggable storage
interface, rich transaction support, and undo.
"""

# The (non-obvious!) choices for the Trove Development Status line:
# Development Status :: 5 - Production/Stable
# Development Status :: 4 - Beta
# Development Status :: 3 - Alpha

classifiers = """\
Development Status :: 5 - Production/Stable
Intended Audience :: Developers
License :: OSI Approved :: Zope Public License
Programming Language :: Python
Topic :: Database
Topic :: Software Development :: Libraries :: Python Modules
Operating System :: Microsoft :: Windows
Operating System :: Unix
"""

import glob
import os
import sys
from distutils.core import setup
from distutils.extension import Extension
from distutils import dir_util
from distutils.core import setup
from distutils.dist import Distribution
from distutils.command.install_lib import install_lib
from distutils.command.build_py import build_py

# distutils doesn't currently support graceful future evolution,
# because all commands are configured by keyword args and distutils
# complains about keywords it doesn't know about.

if not "depends" in Extension.__init__.func_code.co_varnames:
    # If it doesn't, create a local replacement that removes depends from the
    # kwargs before calling the regular constructor.
    _Extension = Extension
    class Extension(_Extension):
        def __init__(self, name, sources, **kwargs):
            if kwargs.has_key("depends"):
                del kwargs["depends"]
            _Extension.__init__(self, name, sources, **kwargs)

def ExtClassExt(name):
    """Return an Extension object for something in ExtensionClass/src."""
    return Extension(name=name, sources=["ExtensionClass/src/%s.c" % name])

exts = [ExtClassExt(name) for name in
        ["ExtensionClass", "Acquisition", "ComputedAttribute", "MethodObject",
         "Missing", "MultiMapping", "Record", "ThreadLock"]]

# Include directories for C extensions
include = ['ExtensionClass/src', 'ZODB']

# Set up dependencies for the BTrees package
base_btrees_depends = [
    "BTrees/BTreeItemsTemplate.c",
    "BTrees/BTreeModuleTemplate.c",
    "BTrees/BTreeTemplate.c",
    "BTrees/BucketTemplate.c",
    "BTrees/MergeTemplate.c",
    "BTrees/SetOpTemplate.c",
    "BTrees/SetTemplate.c",
    "BTrees/TreeSetTemplate.c",
    "BTrees/sorters.c",
    ]

_flavors = {"O": "object", "I": "int"}

KEY_H = "BTrees/%skeymacros.h"
VALUE_H = "BTrees/%svaluemacros.h"

def BTreeExtension(flavor):
    key = flavor[0]
    value = flavor[1]
    name = "BTrees._%sBTree" % flavor
    sources = ["BTrees/_%sBTree.c" % flavor]
    kwargs = {"include_dirs": include}
    if flavor != "fs":
        kwargs["depends"] = (base_btrees_depends + [KEY_H % _flavors[key],
                                                    VALUE_H % _flavors[value]])
    if key != "O":
        kwargs["define_macros"] = [('EXCLUDE_INTSET_SUPPORT', None)]
    return Extension(name, sources, **kwargs)

exts += [BTreeExtension(flavor) for flavor in ["OO", "IO", "OI", "II", "fs"]]

cPersistence = Extension(name = 'ZODB.cPersistence',
                         include_dirs = include,
                         sources= ['ZODB/cPersistence.c']
                         )

cPickleCache = Extension(name = 'ZODB.cPickleCache',
                         include_dirs = include,
                         sources= ['ZODB/cPickleCache.c']
                         )

TimeStamp = Extension(name = 'ZODB.TimeStamp',
                      define_macros = [('USE_EXTENSION_CLASS', 1)],
                      include_dirs = include,
                      sources= ['ZODB/TimeStamp.c']
                      )

coptimizations = Extension(name = 'ZODB.coptimizations',
                           include_dirs = include,
                           sources= ['ZODB/coptimizations.c']
                           )

winlock = Extension(name = 'ZODB.winlock',
                    include_dirs = include,
                    sources = ['ZODB/winlock.c']
                    )

bsddbhelper = Extension(name = 'BDBStorage._helper',
                        sources = ['BDBStorage/_helper.c'])

exts += [cPersistence, cPickleCache, TimeStamp, coptimizations, winlock,
         bsddbhelper]

packages = ["BDBStorage", "BDBStorage.tests",
            "BTrees", "BTrees.tests",
            "ZEO", "ZEO.auth", "ZEO.zrpc", "ZEO.tests",
            "ZODB", "ZODB.tests",
            "Persistence",
            "ThreadedAsync",
            "zLOG", "zLOG.tests",
            "zdaemon", "zdaemon.tests",
            "ZopeUndo", "ZopeUndo.tests",
            "ZConfig", "ZConfig.tests",
            "ZConfig.tests.library", "ZConfig.tests.library.widget",
            "ZConfig.tests.library.thing",
            ]
package_dir = {'BDBStorage': 'BDBStorage'}

scripts = ["Tools/fsdump.py",
           "Tools/fsrefs.py",
           "Tools/fstail.py",
           "Tools/fstest.py",
           "Tools/repozo.py",
           "Tools/zeopack.py",
           "ZConfig/scripts/zconfig",
           "ZEO/runzeo.py",
           "ZEO/mkzeoinst.py",
           "zdaemon/zdrun.py",
           "zdaemon/zdctl.py",
           ]

def copy_other_files(cmd, outputbase):
    for inputdir in [
        os.path.join("ZConfig", "tests", "input"),
        os.path.join("ZConfig", "tests", "library"),
        os.path.join("ZConfig", "tests", "library", "thing"),
        os.path.join("ZConfig", "tests", "library", "thing", "extras"),
        os.path.join("ZConfig", "tests", "library", "widget"),
        "ZEO",
        "ZODB",
        "zdaemon",
        os.path.join("zdaemon", "tests"),
        "zLOG",
        ]:
        outputdir = os.path.join(outputbase, inputdir)
        if not os.path.exists(outputdir):
            dir_util.mkpath(outputdir)
        for pattern in ("*.conf", "*.xml", "*.txt", "*.sh"):
            for fn in glob.glob(os.path.join(inputdir, pattern)):
                cmd.copy_file(fn, os.path.join(outputbase, fn))

class MyLibInstaller(install_lib):
    """Custom library installer, used to put hosttab in the right place."""
    # We use the install_lib command since we need to put hosttab
    # inside the library directory.  This is where we already have the
    # real information about where to install it after the library
    # location has been set by any relevant distutils command line
    # options.

    def run(self):
        install_lib.run(self)
        copy_other_files(self, self.install_dir)

class MyPyBuilder(build_py):
    def build_packages(self):
        build_py.build_packages(self)
        copy_other_files(self, self.build_lib)

class MyDistribution(Distribution):
    # To control the selection of MyLibInstaller and MyPyBuilder, we
    # have to set it into the cmdclass instance variable, set in
    # Distribution.__init__().

    def __init__(self, *attrs):
        Distribution.__init__(self, *attrs)
        self.cmdclass['build_py'] = MyPyBuilder
        self.cmdclass['install_lib'] = MyLibInstaller

doclines = __doc__.split("\n")

setup(name="ZODB3",
      version="3.2.8",
      maintainer="Zope Corporation",
      maintainer_email="zodb-dev@zope.org",
      url = "http://www.zope.org/Wikis/ZODB/FrontPage",
      packages = packages,
      package_dir = package_dir,
      ext_modules = exts,
      headers = ['ExtensionClass/src/ExtensionClass.h', 'ZODB/cPersistence.h'],
      license = "http://www.zope.org/Resources/ZPL",
      platforms = ["any"],
      description = doclines[0],
      classifiers = filter(None, classifiers.split("\n")),
      long_description = "\n".join(doclines[2:]),
      distclass = MyDistribution,
      scripts = scripts,
      )
